"use strict";
/**
 * @file Ce fichier contient la classe de l'API du LDAP qui gère les opérations sur les groupes. Elle est destinée à être exportée pour être utilisée par les resolvers.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var config_1 = require("../internal/config");
exports.userData = config_1.userData;
var basics_1 = require("../internal/basics");
var tools_1 = require("../internal/tools");
var User = /** @class */ (function () {
    /**
     * @memberof LDAP
     * @class User
     * @classdesc Cette classe est une des deux classes exportables permettant de faire des opérations sur les utilisateurs.
     * @summary Constructeur vide.
     */
    function User() {
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui renvoit les infos de base relatives à un utilisateur particulier.
     * @desc Cette fonction utilise {@link Tools.peek} avec l'interface {@link userData}.
     * @arg {string} uid - Identifiant de l'utilisateur, supposé valide.
     * @return {Promise(userData)} Informations recueillies au format {@link userData}.
     * @static
     * @async
     */
    User.peek = function (uid) {
        return __awaiter(this, void 0, void 0, function () {
            var data, _a, _b, _i, cat, _c, _d, err_1;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _e.trys.push([0, 6, , 7]);
                        return [4 /*yield*/, tools_1.Tools.peek("user", uid, config_1.userData)];
                    case 1:
                        data = _e.sent();
                        _a = [];
                        for (_b in config_1.categories)
                            _a.push(_b);
                        _i = 0;
                        _e.label = 2;
                    case 2:
                        if (!(_i < _a.length)) return [3 /*break*/, 5];
                        cat = _a[_i];
                        _c = data;
                        _d = cat;
                        return [4 /*yield*/, basics_1.Basics.searchSingle("group", config_1.ldapConfig.group[cat], "*", config_1.ldapConfig.group[cat] + "=" + uid)];
                    case 3:
                        _c[_d] = _e.sent();
                        _e.label = 4;
                    case 4:
                        _i++;
                        return [3 /*break*/, 2];
                    case 5: return [2 /*return*/, data];
                    case 6:
                        err_1 = _e.sent();
                        throw "Error while peeking a user.";
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui retrouve les uid des paxs validant les critères de recherche. Utiliser {@link User.peek} au cas par cas après pour obtenir les vraies infos.
     * @desc Cette fonction utilise {@link Tools.search}.
     * @arg {userData} data - Dictionnaire contenant les données nécessaires à la recherche. Les valeurs sont celles entrées par l'utilisateur et sont par hypothèse
     * comme des sous-parties compactes des valeurs renvoyées. Tous les champs ci-dessous peuvent être indifféremment des listes (par exemple pour chercher un membre
     * de plusieurs groupes) ou des éléments isolés. Si un champ n'est pas pertinent, le mettre à '' ou undefined.
     * @return {Promise(string[])} gids des profils qui "match" les critères proposés.
     * @static
     * @async
     */
    User.search = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                try {
                    return [2 /*return*/, tools_1.Tools.search("user", data)];
                }
                catch (err) {
                    throw "Erreur lors de la recherche approximative d'un utilisateur.";
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui créé un nouvel utilisateur dans le LDAP.
     * @desc Appelle {@link LDAP.add} bien sûr, mais aussi {@link Tools.add} pour gérer les groupes du nouvel utilisateur.
     * @arg {userData} data - Dictionnaire des informations utilisateurs. Des erreurs peuvent apparaître si tous les champs ne sont pas remplis.
     * Cette application permet de rejoindre des groupes en masse pour toute catégorie.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    User.create = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var vals, uid, key_att, _loop_1, key_att, vals3, _a, _b, err_2, _i, categories_1, cat, _c, _d, gid;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        vals = {};
                        // uid de base généré à partir de nom et prénom, plus potentiellement promo et un offset
                        // MEF mélange de Promise et de fonction standard
                        try {
                            tools_1.Tools.generateUid(data['givenName'], data['lastName'], data['birthdate']).then(function (id) { vals[config_1.ldapConfig.key_id] = id; });
                        }
                        catch (err) {
                            throw "Erreur lors de la génération d'un hruid pour un nouvel utilisateur.";
                        }
                        uid = vals[config_1.ldapConfig.key_id];
                        // Génère une erreur si un champ n'est pas rempli
                        for (key_att in data) {
                            // Ecriture de toutes les valeurs uniques
                            if (!Array.isArray(data[key_att])) {
                                vals[config_1.ldapConfig.user[key_att]] = data[key_att];
                            }
                        }
                        return [4 /*yield*/, basics_1.Basics.add("user", vals)];
                    case 1:
                        // Appel à la fonction de base
                        if (!(_e.sent())) {
                            throw "Erreur de l'ajout de la feuille à l'arbre utilisateur.";
                        }
                        _loop_1 = function (key_att) {
                            // Modifications multiples pour avoir plusieurs champs de même type ; boucle sur les attributs multiples
                            if (Array.isArray(data[key_att])) {
                                // On rajoute chaque valeur en entrée
                                data[key_att].forEach(function (val) {
                                    var vals2 = {};
                                    vals2[config_1.ldapConfig.user[key_att]] = val;
                                    basics_1.Basics.change("user", uid, "add", vals2).then(function (res) {
                                        if (!res) {
                                            throw "Erreur lors de l'ajout d'une valeur pour un champ à valeurs multiples à la feuille du nouvel utilisateur.";
                                        }
                                    });
                                });
                            }
                        };
                        for (key_att in data) {
                            _loop_1(key_att);
                        }
                        vals3 = {};
                        // ldapConfiguration du mot de passe utilisateur
                        // Le préfixe {CRYPT} signifie que le mdp est hashé dans OpenLDAP voir : https://www.openldap.org/doc/admin24/security.html 
                        vals3[config_1.ldapConfig.user['password']] = "{CRYPT}" + data['password'];
                        // Ecriture d'un surnom s'il y a lieu
                        if ((data['nickname'] != undefined) && (data['nickname'] != '')) {
                            vals3[config_1.ldapConfig.user['nickname']] = data['nickname'];
                        }
                        _e.label = 2;
                    case 2:
                        _e.trys.push([2, 4, , 5]);
                        // Génération id aléatoire unique
                        _a = vals3;
                        _b = config_1.ldapConfig.user['id'];
                        return [4 /*yield*/, tools_1.Tools.generateId(config_1.ldapConfig.user['id'], "user")];
                    case 3:
                        // Génération id aléatoire unique
                        _a[_b] = _e.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        err_2 = _e.sent();
                        throw "Erreur lors de la génération d'un id numérique pour un nouvel utilisateur.";
                    case 5:
                        // Code root
                        vals3[config_1.ldapConfig.user['cleanFullName']] = data['fullName'].replace(':', ';').toLowerCase().normalize('UFD');
                        return [4 /*yield*/, basics_1.Basics.change("user", uid, "add", vals3)];
                    case 6:
                        // Inscription des valeurs calculées
                        if (!(_e.sent())) {
                            throw "Erreur lors de l'ajout des valeurs calculées à la feuille du nouvel utilisateur.";
                        }
                        ["posixAccount", "shadowAccount", "brUser"].forEach(function (cst) {
                            var val3 = {};
                            vals3[config_1.ldapConfig.user['class']] = cst;
                            basics_1.Basics.change("user", uid, "add", vals3).then(function (res) {
                                if (!res) {
                                    throw "Erreur lors de l'ajout d'une valeur constante à la feuille du nouvel utilisateur.";
                                }
                            });
                        });
                        // Ajout dans les groupes à la catégorie voulue
                        for (_i = 0, categories_1 = config_1.categories; _i < categories_1.length; _i++) {
                            cat = categories_1[_i];
                            for (_c = 0, _d = data[cat]; _c < _d.length; _c++) {
                                gid = _d[_c];
                                tools_1.Tools.add(uid, gid, cat);
                            }
                        }
                        return [2 /*return*/, true];
                }
            });
        });
    };
    //------------------------------------------------------------------------------------------------------------------------
    // Fonctions de suppression TBT
    //------------------------------------------------------------------------------------------------------------------------
    /**
     * @memberof LDAP
     * @summary Fonction qui supprime un utilisateur du LDAP.
     * @desc Cette fonction commence par gérer les groupes du membre puis le supprime entièrement.
     * Appelle {@link LDAP.clear} bien sûr, mais aussi {@link Tools.remove} pour gérer les groupes de l'utilisateur sortant.
     * @arg {string} uid - uid de la victime
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    User["delete"] = function (uid) {
        return __awaiter(this, void 0, void 0, function () {
            var profil, _loop_2, _i, categories_2, cat, err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, User.peek(uid)];
                    case 1:
                        profil = _a.sent();
                        _loop_2 = function (cat) {
                            profil[config_1.ldapConfig.user[cat]].forEach(function (gid) {
                                return __awaiter(this, void 0, void 0, function () {
                                    var lm;
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0: return [4 /*yield*/, tools_1.Tools.get(gid, "group", cat)];
                                            case 1:
                                                lm = _a.sent();
                                                if (!lm.includes(uid)) return [3 /*break*/, 3];
                                                return [4 /*yield*/, basics_1.Basics.change("group", gid, "del", config_1.ldapConfig.group[cat])];
                                            case 2:
                                                // Supprime tous les membres
                                                if (!(_a.sent())) {
                                                    throw "Erreur lors de la suppression de tous les membres du groupe.";
                                                }
                                                // Les rajoute un par un, sauf pour le supprimé
                                                lm.forEach(function (id) {
                                                    if (id != uid) {
                                                        tools_1.Tools.add(id, gid, cat).then(function (res) {
                                                            if (!res) {
                                                                throw "Erreur lors du ré-ajout d'un autre membre";
                                                            }
                                                        });
                                                    }
                                                });
                                                _a.label = 3;
                                            case 3: return [2 /*return*/];
                                        }
                                    });
                                });
                            });
                        };
                        for (_i = 0, categories_2 = config_1.categories; _i < categories_2.length; _i++) {
                            cat = categories_2[_i];
                            _loop_2(cat);
                        }
                        return [3 /*break*/, 3];
                    case 2:
                        err_3 = _a.sent();
                        throw "Erreur lors de l'obtention des informations de l'utilisateur à supprimer.";
                    case 3:
                        // Elimination
                        if (!basics_1.Basics.clear("user", uid)) {
                            throw "Erreur lors de la suppression de l'utilisateur.";
                        }
                        return [2 /*return*/, true];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui édite un utilisateur existant dans le LDAP.
     * @desc Appelle simplement {@link Tools.edit}. Sans effet sur les groupes de l'utilisateur concerné.
     * @arg {userData} data - Dictionnaire des informations utilisateurs
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    User.edit = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                try {
                    return [2 /*return*/, tools_1.Tools.edit("user", data)];
                }
                catch (err) {
                    throw "Erreur lors de la modification d'un utilisateur.";
                }
                return [2 /*return*/];
            });
        });
    };
    return User;
}());
exports.User = User;
