"use strict";
/**
 * @file Ce fichier regroupe les fonctions fondamentales aux interactions avec le LDAP.
 * C'est ici que tout le filtrage est opéré, au plus bas niveau.
 * Toutes les fonctions écrites ici sont asynchrones et renvoient des Promises ce qui nécessite de les appeler avec la synthaxe
 * un peu particulière `f(args).then(res => ...)` pour exploiter leur résultat.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var ldapjs_1 = require("ldapjs");
// Toutes les entrées utilisateur sont escapées par sécurité
var ldap_escape_1 = require("ldap-escape");
// Fichier de ldapConfig du ldap
var config_1 = require("./config");
// Connection au serveur LDAP avec des temps de timeout arbitraires
var client = ldapjs_1["default"].createClient({ url: config_1.ldapConfig.server });
//------------------------------------------------------------------------------------------------------------------------
// Fonctions de base agissant sur le LDAP
//------------------------------------------------------------------------------------------------------------------------
var Basics = /** @class */ (function () {
    /**
     * @class Basics
     * @classdesc Cette classe est la brique de base du fichier tout entier puisqu'elle contient les functions qui agissent directement sur le LDAP.
     * @memberof LDAP
     * @summary Constructeur vide.
     */
    function Basics() {
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à s'identifier sur le LDAP.
     * @desc Assez important en terme de sécurité, de gestion de conflit, et de droit d'accès. Méthode ldapjs
     * (voir [`Client API`](http://ldapjs.org/client.html) méthode bind).
     * @arg {string} dn - Nom de domaine ; identifiant de l'utilisateur cherchant à se connecter
     * @arg {string} password - Mot de passe de l'utilisateur cherchant à se connecter
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    Basics.bind = function (dn, password) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                // Escape DN as everywhere in this file, but password is taken as is
                client.bind(dn, password, function (res) {
                    // Gestion erreur
                    try {
                        res;
                    }
                    catch (err) {
                        throw "Erreur lors de la connection au LDAP.";
                    }
                });
                // End with a boolean
                return [2 /*return*/, true];
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à s'identifier sur le LDAP avec plein pouvoirs.
     * @desc Appelle {@link bind} avec un utilisateur tout puissant.
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    Basics.adminBind = function () {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, Basics.bind(config_1.credentialsLdapConfig.dn, config_1.credentialsLdapConfig.password)];
        }); });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à se déconnecter du LDAP.
     * @desc Assez important en terme de sécurité, de gestion de conflit, et de droit d'accès.
     * Fait appel à {@link Basics.bind} avec deux champs vides.
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    Basics.unbind = function () {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/, Basics.bind("", "")];
        }); });
    };
    /**
     * @callback entryHandler
     * @arg entry {*} - Convoluted ldap.js search result object
     */
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et modifie une liste pour y insérer les valeurs trouvées.
     * @desc Cette fonction utilise ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode search). Cette fonction fait une demande au LDAP
     * qu'elle filtre selon un schéma prédéfini dans `filter` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string[]} attributes - Attributs à renvoyer
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @arg {entryHandler} handler - Wrapper pour gérer les requêtes simples ou multiples
     * @return {void} Utilise handler pour gérer ses résultats au fur et à mesure
     * @static
     * @async
     */
    Basics.search = function (domain, attributes, id, filter, handler) {
        Basics.adminBind();
        var dn = "";
        if (id != null) {
            dn += config_1.ldapConfig.key_id + '=' + ldap_escape_1["default"].dn("${txt}", { txt: id }) + ',';
        }
        dn += config_1.ldapConfig.dn[domain];
        // Interrogation LDAP selon filter
        var promise = new Promise(function (resolve, reject) {
            client.search(dn, {
                "scope": "sub",
                "filter": filter,
                "attributes": attributes
            }, function (err, res) {
                // Gestion erreur ; pb car pas simple true / autre en sortie
                if (err) {
                    throw "Erreur lors de la recherche sur le LDAP.";
                }
                else {
                    // Dès que la recherche renvoit une entrée, on stocke les attributs qui nous intéresse
                    res.on('searchEntry', function (entry) { handler(entry); });
                    // Si la recherche renvoie une erreur, on renvoit
                    res.on('error', function (resErr) { throw resErr; });
                    // Quand la recherche est finie on se déconnecte
                    res.on('end', function (_) { Basics.unbind(); resolve(); });
                }
            });
        });
        return promise;
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et renvoit une liste de valeurs trouvées.
     * @desc Cette fonction utilise {@link LDAP.search} directement.
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} attribute - Attribut unique à renvoyer
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @return {Promise(string[])} Résultats de la recherche ; soit une liste de valeurs d'attributs,
     * soit une liste de dictionnaires si on veut plus d'un attribut (les clés du dictionnaire sont celles du LDAP)
     * @static
     * @async
     */
    Basics.searchSingle = function (domain, attribute, id, filter) {
        if (id === void 0) { id = null; }
        if (filter === void 0) { filter = "(objectClass=*)"; }
        return __awaiter(this, void 0, void 0, function () {
            var vals;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        vals = [];
                        return [4 /*yield*/, Basics.search(domain, [attribute], id, filter, function (entry) {
                                // Cas un seul attribut où le résultat est une liste directement
                                vals.push(entry.object[attribute]);
                            })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, vals];
                }
            });
        });
    };
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et renvoit les valeurs trouvées.
     * @desc Cette fonction utilise ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode search). Cette fonction fait une demande au LDAP
     * qu'elle filtre selon un schéma prédéfini dans `filter` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string[]} attributes - Liste des attributs qui figureront dans le résultat final ; peut aussi être un seul élément
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @return {Promise(Array<dic>)} Résultats de la recherche ; soit une liste de valeurs d'attributs,
     * soit une liste de dictionnaires si on veut plus d'un attribut (les clés du dictionnaire sont celles du LDAP)
     * @static
     * @async
     */
    Basics.searchMultiple = function (domain, attributes, id, filter) {
        if (id === void 0) { id = null; }
        if (filter === void 0) { filter = "(objectClass=*)"; }
        return __awaiter(this, void 0, void 0, function () {
            var vals;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        vals = [];
                        return [4 /*yield*/, Basics.search(domain, attributes, id, filter, function (entry) {
                                // Cas plusieurs attributs donc résultat dictionnaire
                                vals.push({});
                                attributes.forEach(function (attribute) {
                                    vals.slice(-1)[0][attribute] = entry.object[attribute];
                                });
                            })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, vals];
                }
            });
        });
    };
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de modifier un élément sur le LDAP. Gestion intelligente de l'appartenance à un binet.
     * @desc Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode modify).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} id - Identifiant unique de la feuille à modifier
     * @arg {"add"|"del"|"replace"} op - Operation à réaliser sur le LDAP. Trois opération sont possibles ; "add", qui rajoute des attributs et qui peut créer des doublons,
     * "del" qui en supprime, et "replace" qui remplace du contenu par un autre.
     * @arg {dic} mod - Dictionnaire contenant les attributs à modifier et les nouvelles valeurs des attributs.
     * @arg {string} mod[key] - Nouvelle valeur de l'attribut key. Une nouvelle valeur vide ("") est équivalent à la suppression de cet attribut.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    Basics.change = function (domain, id, op, mod) {
        return __awaiter(this, void 0, void 0, function () {
            var dn;
            return __generator(this, function (_a) {
                Basics.adminBind();
                dn = config_1.ldapConfig.key_id + '=' + id + ',';
                dn += config_1.ldapConfig.dn[domain];
                // Modification LDAP selon dn fourni en argument (pourrait prendre une liste de Changes)
                client.modify(ldap_escape_1["default"].dn("${txt}", { txt: dn }), new ldapjs_1["default"].Change({
                    operation: op,
                    modification: mod
                }), function (err) {
                    throw "Erreur lors d'une opération de modification sur le LDAP.";
                });
                Basics.unbind();
                return [2 /*return*/, true];
            });
        });
    };
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un élément sur le LDAP.
     * @desc  Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode add).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {Object.<string, string>} vals - Dictionnaire contenant les valeurs à créer (contient un champ en ldapConfig.key_id)
     * @arg {Object} vals[key] - Nouvelle valeur pour le champ key
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon.
     * @static
     * @async
     */
    Basics.add = function (domain, vals) {
        return __awaiter(this, void 0, void 0, function () {
            var dn;
            return __generator(this, function (_a) {
                Basics.adminBind();
                dn = config_1.ldapConfig.key_id + "=" + vals[config_1.ldapConfig.key_id];
                dn += config_1.ldapConfig.dn[domain];
                // Ajout LDAP selon la ldapConfiguration en argument
                client.add(ldap_escape_1["default"].dn("${txt}", { txt: dn }), vals, function (err) {
                    throw "Erreur lors d'une opération d'ajout sur le LDAP.";
                });
                Basics.unbind();
                return [2 /*return*/, true];
            });
        });
    };
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer une feuille du LDAP.
     * @desc Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode del).
     * Elle est différente de modify avec "del" car elle affecte directement une feuille et pas un attribut.
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} id - Identifiant unique de la cible
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @static
     * @async
     */
    Basics.clear = function (domain, id) {
        return __awaiter(this, void 0, void 0, function () {
            var dn;
            return __generator(this, function (_a) {
                Basics.adminBind();
                dn = config_1.ldapConfig.key_id + '=' + id + ',';
                dn += config_1.ldapConfig.dn[domain];
                // Suppression LDAP
                client.del(ldap_escape_1["default"].dn("${txt}", { txt: dn }), function (err) {
                    throw "Erreur lors d'une opération de suppression sur le LDAP.";
                });
                Basics.unbind();
                return [2 /*return*/, true];
            });
        });
    };
    return Basics;
}());
exports.Basics = Basics;
